@@ -30,9 +30,10 @@ import bisect
 from glue import iterutils
 from glue import segments
 from glue.ligolw import lsctables
-from pylal import ligolw_burca2
 from pylal import ligolw_thinca
+from pylal import snglcoinc
 from pylal.date import XLALUTCToGPS
+import lal
 import time
 
 
@@ -50,7 +51,7 @@ import time
 #
 
 
-allowed_instrument_combos = (frozenset(("H1", "H2", "L1")), frozenset(("H1", "L1", "V1")), frozenset(("H1", "L1")), frozenset(("H1", "V1")), frozenset(("L1", "V1")), frozenset(("H1H2", "L1")), frozenset(("H1H2", "L1", "V1")))
+allowed_instrument_combos = frozenset([frozenset(("H1", "H2", "L1")), frozenset(("H1", "L1", "V1")), frozenset(("H1", "L1")), frozenset(("H1", "V1")), frozenset(("L1", "V1")), frozenset(("H1H2", "L1")), frozenset(("H1H2", "L1", "V1")), frozenset(("E1", "E2")), frozenset(("E1", "E3")), frozenset(("E2", "E3")), frozenset(("E1", "E2", "E3"))])
 
 
 #
@@ -164,19 +165,16 @@ ligolw_thinca.InspiralEventList = InspiralEventList
 
 
 class StreamThinca(object):
-	def __init__(self, coincidence_threshold, thinca_interval = 50.0, coinc_params_distributions = None, likelihood_params_func = None, trials_table = None, sngls_snr_threshold = None):
+	def __init__(self, coincidence_threshold, thinca_interval = 50.0, sngls_snr_threshold = None):
 		self._xmldoc = None
-		self.thinca_interval = thinca_interval
-		self.set_likelihood_data(coinc_params_distributions, likelihood_params_func)
+		self.thinca_interval = thinca_interval	# seconds
 		self.last_coincs = {}
-		self.trials_table = trials_table
 		self.sngls_snr_threshold = sngls_snr_threshold
 		self.sngl_inspiral_table = None
+		self.ln_likelihood_func = None
+		self.ln_likelihood_params_func = None
 
-		# when using the normal coincidence function from
-		# ligolw_thinca this is the e-thinca parameter.  when using
-		# a \Delta t only coincidence test it's the \Delta t window
-		# not including the light travel time
+		# the \Delta t window not including the light travel time
 		self.coincidence_threshold = coincidence_threshold
 
 		# upper boundary of interval spanned by last invocation
@@ -187,17 +185,16 @@ class StreamThinca(object):
 		self.ids = set()
 
 
-	def set_likelihood_data(self, coinc_params_distributions, likelihood_params_func):
-		if coinc_params_distributions is not None:
-			assert likelihood_params_func is not None
-			self.likelihood_func = ligolw_burca2.LikelihoodRatio(coinc_params_distributions)
-		else:
-			assert likelihood_params_func is None
-			self.likelihood_func = None
-		self.likelihood_params_func = likelihood_params_func
+	def set_coinc_params_distributions(self, coinc_params_distributions):
+		self.ln_likelihood_func = snglcoinc.LnLikelihoodRatio(coinc_params_distributions)
+		self.ln_likelihood_params_func = coinc_params_distributions.coinc_params
+	def del_coinc_params_distributions(self):
+		self.ln_likelihood_func = None
+		self.ln_likelihood_params_func = None
+	coinc_params_distributions = property(None, set_coinc_params_distributions, del_coinc_params_distributions, "ThincaCoincParamsDistributions instance with which to compute likelihood ratio values.")
 
 
-	def add_events(self, xmldoc, process_id, events, boundary, FAP = None):
+	def add_events(self, xmldoc, process_id, events, boundary, fapfar = None):
 		# invalidate the coinc extractor in case all that follows
 		# is a no-op
 		self.last_coincs = {}
@@ -209,7 +206,7 @@ class StreamThinca(object):
 		# we need to ensure we have a Table subclass, not a DBTable
 		# subclass
 		if self.sngl_inspiral_table is None:
-			self.sngl_inspiral_table = lsctables.New(lsctables.SnglInspiralTable, lsctables.table.get_table(xmldoc, lsctables.SnglInspiralTable.tableName).columnnames)
+			self.sngl_inspiral_table = lsctables.New(lsctables.SnglInspiralTable, lsctables.SnglInspiralTable.get_table(xmldoc).columnnames)
 			# so we can watch for it changing
 			assert self._xmldoc is None
 			self._xmldoc = xmldoc
@@ -226,16 +223,16 @@ class StreamThinca(object):
 		# no new events
 		if not events:
 			return []
-		return self.run_coincidence(xmldoc, process_id, boundary, FAP = FAP)
+		return self.run_coincidence(xmldoc, process_id, boundary, fapfar = fapfar)
 
 
-	def run_coincidence(self, xmldoc, process_id, boundary, FAP = None):
+	def run_coincidence(self, xmldoc, process_id, boundary, fapfar = None):
 		# safety check
 		assert xmldoc is self._xmldoc
 
 		# stay this far away from the boundaries of the available
 		# triggers
-		coincidence_back_off = max(abs(offset) for offset in lsctables.table.get_table(xmldoc, lsctables.TimeSlideTable.tableName).getColumnByName("offset"))
+		coincidence_back_off = max(abs(offset) for offset in lsctables.TimeSlideTable.get_table(xmldoc).getColumnByName("offset"))
 
 		# check that we've accumulated thinca_interval seconds, and
 		# that .add_events() has been called with some events since
@@ -260,10 +257,10 @@ class StreamThinca(object):
 		coinc_inspiral_table = lsctables.New(lsctables.CoincInspiralTable)
 
 		# replace tables with our versions
-		real_sngl_inspiral_table = lsctables.table.get_table(xmldoc, lsctables.SnglInspiralTable.tableName)
-		real_coinc_event_map_table = lsctables.table.get_table(xmldoc, lsctables.CoincMapTable.tableName)
-		real_coinc_event_table = lsctables.table.get_table(xmldoc, lsctables.CoincTable.tableName)
-		real_coinc_inspiral_table = lsctables.table.get_table(xmldoc, lsctables.CoincInspiralTable.tableName)
+		real_sngl_inspiral_table = lsctables.SnglInspiralTable.get_table(xmldoc)
+		real_coinc_event_map_table = lsctables.CoincMapTable.get_table(xmldoc)
+		real_coinc_event_table = lsctables.CoincTable.get_table(xmldoc)
+		real_coinc_inspiral_table = lsctables.CoincInspiralTable.get_table(xmldoc)
 		xmldoc.childNodes[-1].replaceChild(self.sngl_inspiral_table, real_sngl_inspiral_table)
 		xmldoc.childNodes[-1].replaceChild(coinc_event_map_table, real_coinc_event_map_table)
 		xmldoc.childNodes[-1].replaceChild(coinc_event_table, real_coinc_event_table)
@@ -290,78 +287,31 @@ class StreamThinca(object):
 			event_comparefunc = event_comparefunc,
 			thresholds = self.coincidence_threshold,
 			ntuple_comparefunc = ntuple_comparefunc,
-			likelihood_func = self.likelihood_func,
-			likelihood_params_func = self.likelihood_params_func
+			likelihood_func = self.ln_likelihood_func,
+			likelihood_params_func = self.ln_likelihood_params_func,
+			# add 10% to coincidence window for safety + the
+			# light-crossing time for the Earth
+			max_dt = 1.1 * self.coincidence_threshold + 2. * lal.REARTH_SI / lal.C_SI
 		)
 
 		# restore .get_effective_snr() method on trigger class
 		ligolw_thinca.SnglInspiral.get_effective_snr = orig_get_effective_snr
 
-		# increment the trials table and possibly assign FAPs
-		# set the live time
-		coinc_event_index = dict((row.coinc_event_id, row) for row in coinc_event_table)
-		gps_time_now = XLALUTCToGPS(time.gmtime())
-		have_incremented_count_below_thresh = False
-		trials_dict = {}
-		for coinc_inspiral_row in coinc_inspiral_table:
-			coinc_event_row = coinc_event_index[coinc_inspiral_row.coinc_event_id]
-			# increment the trials table
-			ifo_set = frozenset(coinc_inspiral_row.get_ifos())
-			# FIXME, don't hard code this.  Think about the mass dimension too.
-			# Add the integer truncation of the trigger time * 2
-			# to a set.  This is effectively like binning the
-			# events by end time in 500 ms bins. This is a way of
-			# extracting the effective number of independent trials
-			# for later
-			trials_dict.setdefault(ifo_set, set()).add(int(float(coinc_inspiral_row.get_end()) * 200))
-
-			# Assign the FAP if requested
-			if FAP is not None:
-				# note FAP should have a reference to the
-				# global trials table read in by in the
-				# marginalized_likelihood file.  This is not
-				# the same as the one updated on the previous
-				# lines!  This trials table is static until the
-				# marginalized likelihood file is read in
-				# again.
-
-				# compute the false-alarm rate without
-				# using the trials-factor rescaling
-				coinc_inspiral_row.combined_far = FAP.far_from_rank(coinc_event_row.likelihood, ifo_set)
-
-				# now that we know this event's un-adjusted
-				# false-alarm rate, adjust the
-				# trials-factor rescaling
-				# FIXME bad!! We only increment the count below
-				# thresh once in this loop as a way to
-				# "cluster" events similar to the gracedb loop
-				# later.  This needs to be tied together
-				# somehow
-				if not have_incremented_count_below_thresh and coinc_inspiral_row.combined_far < self.trials_table[ifo_set].thresh:
-					self.trials_table[ifo_set].count_below_thresh += 1
-					have_incremented_count_below_thresh = True
-
-				# now re-compute the false-alarm rate, this
-				# time using the trials-factor rescaling
-				coinc_inspiral_row.combined_far = FAP.far_from_rank(coinc_event_row.likelihood, ifo_set, scale = True)
-
-				# compute the false-alarm probability, too,
-				# now that we know the latest the
-				# trials-factor scaling
-				coinc_inspiral_row.false_alarm_rate = FAP.fap_from_rank(coinc_event_row.likelihood, ifo_set)
+		# assign the FAP and FAR if provided with the data to do so
+		if fapfar is not None:
+			coinc_event_index = dict((row.coinc_event_id, row) for row in coinc_event_table)
+			gps_time_now = XLALUTCToGPS(time.gmtime())
+			for coinc_inspiral_row in coinc_inspiral_table:
+				likelihood_ratio = coinc_event_index[coinc_inspiral_row.coinc_event_id].likelihood
+				coinc_inspiral_row.combined_far = fapfar.far_from_rank(likelihood_ratio)
+				# FIXME:  add a proper column to store this in
+				coinc_inspiral_row.false_alarm_rate = fapfar.fap_from_rank(likelihood_ratio)
 
 				# abuse minimum_duration column to store
 				# the latency.  NOTE:  this is nonsensical
 				# unless running live
 				coinc_inspiral_row.minimum_duration = float(gps_time_now - coinc_inspiral_row.get_end())
 
-		# Update the trials table from the independent trials calculated above
-		for ifo_set, trigger_times in trials_dict.items():
-			try:
-				self.trials_table[ifo_set].count += len(trigger_times)
-			except KeyError:
-				self.trials_table[ifo_set].count = len(trigger_times)
-
 		# construct a coinc extractor from the XML document while
 		# the tree still contains our internal table objects
 		self.last_coincs = ligolw_thinca.sngl_inspiral_coincs(xmldoc)
@@ -410,7 +360,7 @@ class StreamThinca(object):
 		return noncoinc_sngls
 
 
-	def flush(self, xmldoc, process_id, FAP = None):
+	def flush(self, xmldoc, process_id, fapfar = None):
 		# invalidate the coinc extractor in case run_coincidence()
 		# is a no-op.
 		self.last_coincs = {}
@@ -418,7 +368,7 @@ class StreamThinca(object):
 		# coincidence.  don't bother unless .add_events() has been
 		# called since the last flush()
 		if self._xmldoc is not None:
-			noncoinc_sngls = self.run_coincidence(xmldoc, process_id, segments.infinity(), FAP = FAP)
+			noncoinc_sngls = self.run_coincidence(xmldoc, process_id, segments.infinity(), fapfar = fapfar)
 		else:
 			noncoinc_sngls = []
 
